@@ -25,6 +25,8 @@ sub clear ($self) {
 # It appears that some browsers limit by bytes, while others limit the number of characters;
 
 sub parse_cookies ( $self, $url, $set_cookie_header ) {
+    $url = P->uri($url) if !ref $url;
+
   COOKIE: for ( $set_cookie_header->@* ) {
         my ( $kvp, @attrs ) = split /;/sm;
 
@@ -36,14 +38,17 @@ sub parse_cookies ( $self, $url, $set_cookie_header ) {
 
         next if $kvp eq q[];
 
+        my $origin_domain_name = $url->host->name;
+
         my $cookie = {
-            domain   => $url->host->name,
+            domain   => $origin_domain_name,
             path     => $url->path->to_string,
             expires  => 0,
             httponly => 0,
             secure   => 0,
         };
 
+        # parse and set key and value
         if ( ( my $idx = index $kvp, q[=] ) != -1 ) {
             $cookie->{name} = substr $kvp, 0, $idx;
 
@@ -55,6 +60,7 @@ sub parse_cookies ( $self, $url, $set_cookie_header ) {
             $cookie->{val} = q[];
         }
 
+        # parse attributes
         for my $attr (@attrs) {
 
             # trim
@@ -77,72 +83,74 @@ sub parse_cookies ( $self, $url, $set_cookie_header ) {
             }
 
             if ( $k eq 'domain' ) {
+                if ( $v ne q[] ) {
 
-                # http://bayou.io/draft/cookie.domain.html
-                #
-                # origin domain - domain from the request
-                # cover domain - domain from cookie attribute
+                    # http://bayou.io/draft/cookie.domain.html
+                    # origin domain - domain from the request
+                    # cover domain - domain from cookie attribute
 
-                # if a cookie's origin domain is an IP, the cover domain must be null
-                next COOKIE if $url->host->is_ip;
+                    # if a cookie's origin domain is an IP, the cover domain must be null
+                    next COOKIE if $url->host->is_ip;
 
-                # remove leading "." from cover domain
-                $v =~ s/\A[.]+//sm;
+                    # parse cover domain
+                    my $cover_domain = P->host($v);
 
-                my $cover_domain = P->host( lc $v );
+                    # a cover domain must not be a IP address
+                    next COOKIE if $cover_domain->is_ip;
 
-                # the cover domain must not be a TLD
-                # As far as cookie handling is concerned, every TLD is a public suffix, even if it's not listed.
-                # For example, "test", "local", "my-fake-tld", etc. cannot be allowed as cover domains.
-                next COOKIE if $cover_domain->is_tld;
+                    my $cover_domain_name = $cover_domain->name;
 
-                # a cover domain must not be a IP address
-                next COOKIE if $cover_domain->is_ip;
+                    # if the origin domain is the same domain
+                    if ( $cover_domain_name eq $origin_domain_name ) {
 
-                # According to RFC_6265, if a cookie's cover domain is a public suffix:
-                # - if the origin domain is the same domain, reset the cover domain to null, then accept the cookie
-                # - otherwise, ignore the cookie entirely
-                if ( $cover_domain->is_pub_suffix ) {
-                    if ( $url->host->name eq $cover_domain->name ) {
-                        next;    # accept cookie
+                        # permit a public suffix domain to specify itself as the cover domain
+                        # ignore cover domain, if cover domain is pub. suffix
+                        next if $cover_domain->is_pub_suffix;
                     }
                     else {
-                        next COOKIE;    # ignore cookie
+                        # the cover domain must not be a TLD, a public suffix, or a parent of a public suffix
+                        next COOKIE if $cover_domain->is_pub_suffix;
+
+                        # the cover domain must cover (be a parent) the origin domain
+                        if ( ( my $idx = index $origin_domain_name, q[.] . $cover_domain_name ) > 0 ) {
+                            next COOKIE if length($origin_domain_name) != 1 + $idx + length $cover_domain_name;
+                        }
+                        else {
+                            next COOKIE;
+                        }
                     }
+
+                    # accept cover domain cookie
+                    $cookie->{domain} = q[.] . $cover_domain_name;
                 }
-
-                # the cover domain must not be a parent of a public suffix
-                #
-                # As far as cookie handling is concerned, parents of a public suffix are public suffixes too, even if they are not listed.
-                # For example, amazonaws.com is not listed as a public suffix, yet it cannot be allowed as cover domain either,
-                # because it is the parent of public suffix compute.amazonaws.com.
-                next COOKIE if $cover_domain->is_pub_suffix_parent;
-
-                # the cover domain must cover (be a substring) the origin domain
-                next COOKIE if substr( $url->host->name, 0 - length $cover_domain->name ) ne $cover_domain->name;
-
-                # accept coveer domain cookie
-                $cookie->{domain} = q[.] . $cover_domain->name;
             }
             elsif ( $k eq 'path' ) {
-                $cookie->{path} = $v;
+                if ( $v ne q[] ) {
+                    $cookie->{path} = $v;
+                }
             }
             elsif ( $k eq 'expires' ) {
-                if ( !$cookie->{expires} ) {    # do not process expires attribute, if expires is already set by expires or max-age
-                    if ( my $expires = eval { P->date->parse($v) } ) {
-                        $cookie->{expires} = $expires->epoch;
-                    }
-                    else {
-                        next COOKIE;            # ignore cookie, invalid expires value
+                if ( $v ne q[] ) {
+                    if ( !$cookie->{expires} ) {    # do not process expires attribute, if expires is already set by expires or max-age
+                        if ( my $expires = eval { P->date->parse($v) } ) {
+                            $cookie->{expires} = $expires->epoch;
+                        }
+                        else {
+                            # ignore cookie if expires value is invalid
+                            next COOKIE;
+                        }
                     }
                 }
             }
             elsif ( $k eq 'max-age' ) {
-                if ( $v =~ /\A\d+\z/sm ) {
-                    $cookie->{expires} = time + $v;
-                }
-                else {
-                    next COOKIE;                # ignore cookie, invalid max-age value
+                if ( $v ne q[] ) {
+                    if ( $v =~ /\A\d+\z/sm ) {
+                        $cookie->{expires} = time + $v;
+                    }
+                    else {
+                        # ignore cookie if max-age value is invalid
+                        next COOKIE;
+                    }
                 }
             }
             elsif ( $k eq 'httponly' ) {
@@ -154,7 +162,7 @@ sub parse_cookies ( $self, $url, $set_cookie_header ) {
         }
 
         if ( $cookie->{expires} && $cookie->{expires} < time ) {
-            delete $self->{cookies}->{ $cookie->{domain} }->{ $cookie->{path} }->{ $cookie->{name} };
+            $self->remove_cookie( $cookie->{domain}, $cookie->{path}, $cookie->{name} );
         }
         else {
             $self->{cookies}->{ $cookie->{domain} }->{ $cookie->{path} }->{ $cookie->{name} } = $cookie;
@@ -165,45 +173,34 @@ sub parse_cookies ( $self, $url, $set_cookie_header ) {
 }
 
 sub get_cookies ( $self, $url ) {
-    my $cookies;
+    state $match_path = sub ( $url_path, $cookie_path ) {
+        return 1 if $cookie_path eq $url_path;
 
-    # origin cookie
-    if ( my $match_cookies = $self->_match_domain( $url->host->name, $url ) ) {
-        push $cookies->@*, $match_cookies->@*;
-    }
+        return 1 if $cookie_path eq q[/];
 
-    # cover cookies
-    # http://bayou.io/draft/cookie.domain.html#Coverage_Model
-    if ( !$url->host->is_ip ) {
-        my @labels = split /[.]/sm, $url->host->name;
+        if ( $url_path =~ /\A\Q$cookie_path\E(.*)/sm ) {
+            my $rest = $1;
 
-        while ( @labels > 1 ) {
-            my $domain = P->host( join q[.], @labels );
+            return 1 if substr( $cookie_path, -1, 1 ) eq q[/];
 
-            last if $domain->is_pub_suffix;
-
-            if ( my $match_cookies = $self->_match_domain( q[.] . $domain->name, $url ) ) {
-                push $cookies->@*, $match_cookies->@*;
-            }
-
-            shift @labels;
+            return 1 if substr( $rest, 0, 1 ) eq q[/];
         }
-    }
 
-    return $cookies;
-}
+        return;
+    };
 
-sub _match_domain ( $self, $domain, $url ) {
-    my $cookies;
+    state $match_domain = sub ( $self, $domain, $domain_cookies, $url ) {
+        my $cookies;
 
-    my $time = time;
+        my $time = time;
 
-    if ( exists $self->{cookies}->{$domain} ) {
-        for my $cookie_path ( keys $self->{cookies}->{$domain}->%* ) {
-            if ( $self->_match_path( $url->path, $cookie_path ) ) {
-                for my $cookie ( values $self->{cookies}->{$domain}->{$cookie_path}->%* ) {
+        for my $cookie_path ( keys $domain_cookies->%* ) {
+            if ( $match_path->( $url->path, $cookie_path ) ) {
+                for my $cookie ( values $domain_cookies->{$cookie_path}->%* ) {
                     if ( $cookie->{expires} && $cookie->{expires} < $time ) {
-                        delete $self->{cookies}->{$domain}->{$cookie_path}->{ $cookie->{name} };
+
+                        # remove expired cookie
+                        $self->remove_cookie( $domain, $cookie_path, $cookie->{name} );
                     }
                     else {
                         next if $cookie->{secure} && !$url->is_secure;
@@ -213,25 +210,60 @@ sub _match_domain ( $self, $domain, $url ) {
                 }
             }
         }
+
+        return $cookies;
+    };
+
+    $url = P->uri($url) if !ref $url;
+
+    my $cookies;
+
+    # origin cookie
+    my $origin_domain_name = $url->host->name;
+
+    if ( my $origin_cookies = $self->{cookies}->{$origin_domain_name} ) {
+        if ( my $match_cookies = $match_domain->( $self, $origin_domain_name, $origin_cookies, $url ) ) {
+            push $cookies->@*, $match_cookies->@*;
+        }
+    }
+
+    # cover cookies
+    # http://bayou.io/draft/cookie.domain.html#Coverage_Model
+    if ( !$url->host->is_ip ) {
+        my @labels = split /[.]/sm, $url->host->name;
+
+        my $origin = 1;
+
+        while ( @labels > 1 ) {
+            my $domain = P->host( join q[.], @labels );
+
+            my $cover_domain_name = q[.] . $domain->name;
+
+            if ( my $cover_cookies = $self->{cookies}->{$cover_domain_name} ) {
+                if ( my $match_cookies = $match_domain->( $self, $cover_domain_name, $cover_cookies, $url ) ) {
+                    push $cookies->@*, $match_cookies->@*;
+                }
+            }
+
+            last if $domain->is_pub_suffix && !$origin;
+
+            $origin = 0;
+
+            shift @labels;
+        }
     }
 
     return $cookies;
 }
 
-sub _match_path ( $self, $url_path, $cookie_path ) {
-    return 1 if $cookie_path eq $url_path;
+sub remove_cookie ( $self, $domain, $path, $name ) {
+    if ( delete $self->{cookies}->{$domain}->{$path}->{$name} ) {
+        delete $self->{cookies}->{$domain}->{$path} if !keys $self->{cookies}->{$domain}->{$path}->%*;
 
-    return 1 if $cookie_path eq q[/];
-
-    if ( $url_path =~ /\A\Q$cookie_path\E(.*)/sm ) {
-        my $rest = $1;
-
-        return 1 if substr( $cookie_path, -1, 1 ) eq q[/];
-
-        return 1 if substr( $rest, 0, 1 ) eq q[/];
+        delete $self->{cookies}->{$domain} if !keys $self->{cookies}->{$domain}->%*;
     }
 
-    return 0;
+    return;
 }
 
 1;
@@ -241,9 +273,13 @@ sub _match_path ( $self, $url_path, $cookie_path ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 27                   | Subroutines::ProhibitExcessComplexity - Subroutine "parse_cookies" with high complexity score (32)             |
+## |    3 |                      | Subroutines::ProhibitExcessComplexity                                                                          |
+## |      | 27                   | * Subroutine "parse_cookies" with high complexity score (38)                                                   |
+## |      | 175                  | * Subroutine "get_cookies" with high complexity score (23)                                                     |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 202, 204             | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## |    3 | 115, 135             | ControlStructures::ProhibitDeepNests - Code structure is deeply nested                                         |
+## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
+## |    3 | 197, 199, 261, 263   | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
