@@ -8,8 +8,8 @@ use Pcore::HTTP::Status;
 use Socket qw[IPPROTO_TCP TCP_NODELAY];
 use Pcore::HTTP::Server::Writer;
 
-has listen => ( is => 'ro', isa => Str,     required => 1 );
-has app    => ( is => 'ro', isa => CodeRef, required => 1 );
+has listen => ( is => 'ro', isa => Str, required => 1 );
+has app => ( is => 'ro', isa => CodeRef | ConsumerOf ['Pcore::HTTP::Server::Router'], required => 1 );
 
 has backlog => ( is => 'ro', isa => Maybe [PositiveOrZeroInt], default => 0 );
 has tcp_no_delay => ( is => 'ro', isa => Bool, default => 0 );
@@ -214,6 +214,8 @@ sub _run_app ( $self, $h, $env ) {
 
     # processing first psgi app response
     if ($@) {
+        warn $@;
+
         $self->_return_xxx( $h, 500 );
     }
     elsif ( ref $res eq 'ARRAY' ) {
@@ -293,6 +295,7 @@ sub _finish_request ( $self, $h, $keep_alive ) {
 }
 
 # TODO add support for different body types, body can be FileHandle or CodeRef or ScalarRef, etc ...
+# TODO convert headers to CamelCase
 sub _write_psgi_response ( $self, $h, $res, $keep_alive, $delayed_body ) {
     my $headers = "HTTP/1.1 $res->[0] " . Pcore::HTTP::Status->get_reason( $res->[0] );
 
@@ -306,24 +309,20 @@ sub _write_psgi_response ( $self, $h, $res, $keep_alive, $delayed_body ) {
     }
 
     # TODO convert headers to CamelCase
-    $headers .= $CRLF . join map { $_->key . q[: ] . $_->value } pairs $res->[1]->@* if $res->[1] && $res->[1]->@*;
+    $headers .= $CRLF . join $CRLF, map { $_->key . q[: ] . $_->value } pairs $res->[1]->@* if $res->[1] && $res->[1]->@*;
 
     if ($delayed_body) {
         $h->push_write( $headers . $CRLF . 'Transfer-Encoding: chunked' . $CRLF . $CRLF );
     }
     else {
         if ( $res->[2] ) {
-            if ( ref $res->[2] eq 'ARRAY' ) {
-                if ( my $body = join q[], $res->[2]->@* ) {
+            if ( ref $res->[2] eq 'SCALAR' ) {
+                $self->_write_buf( $h, \( $headers . $CRLF . 'Content-Length: ' . length( $res->[2]->$* ) . $CRLF . $CRLF . $res->[2]->$* ) );
+            }
+            elsif ( ref $res->[2] eq 'ARRAY' ) {
+                my $body = join q[], $res->[2]->@*;
 
-                    # has body
-                    $self->_write_buf( $h, \( $headers . $CRLF . 'Content-Length: ' . length($body) . $CRLF . $CRLF . $body ) );
-                }
-                else {
-
-                    # body is empty
-                    $self->_write_buf( $h, \( $headers . $CRLF . $CRLF ) );
-                }
+                $self->_write_buf( $h, \( $headers . $CRLF . 'Content-Length: ' . length($body) . $CRLF . $CRLF . $body ) );
             }
             else {
 
@@ -366,9 +365,9 @@ sub _write_buf ( $self, $h, $buf_ref ) {
 ## |======+======================+================================================================================================================|
 ## |    3 | 141                  | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 165, 296             | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
+## |    3 | 165, 299             | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 225                  | ErrorHandling::RequireCheckingReturnValueOfEval - Return value of eval not tested                              |
+## |    3 | 227                  | ErrorHandling::RequireCheckingReturnValueOfEval - Return value of eval not tested                              |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
 ## |    1 | 93                   | CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
